package com.putnam.jboss.issues;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.hornetq.api.core.management.ObjectNameBuilder;
import org.hornetq.api.jms.management.JMSQueueControl;

/**
 * Test harness for issue http://community.jboss.org/thread/153918.
 * @author aengineer
 *
 */
public class DuplicateDetectionIssue {
    
    private JMXConnector jmxc = null;
    
    /*
     * HornetQ
     */
    private final static String JNDI_FACTORY ="org.jnp.interfaces.NamingContextFactory";
    private final static String PROVIDER_URL = "jnp://richmond2.putnaminv.com:6142";
    private final static String JNDI_PRINCIPAL = "";
    private final static String JNDI_PASSWORD = "";
    private final static String JNDI_URL_PKGS = "org.jboss.naming:org.jnp.interfaces";
    private final static String DEST_LOOKUP_NAME = "/queue/ExampleQueue";
    private final static String CF_LOOKUP_NAME = "/ConnectionFactory";
    private final static String DEST_NAME = "ExampleQueue";
    private final static String BROKER_USER = "testuser";
    private final static String BROKER_PASSWORD = "testuser";
    private final static String JMX_URL = "service:jmx:rmi:///jndi/rmi://richmond2:7142/jmxrmi";
    
    
    public static Hashtable<String, String> JNDIEnv = new Hashtable<String, String>();
    static {
        JNDIEnv.put(Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
        JNDIEnv.put(Context.PROVIDER_URL, PROVIDER_URL);
        JNDIEnv.put(Context.SECURITY_PRINCIPAL, JNDI_PRINCIPAL);
        JNDIEnv.put(Context.SECURITY_CREDENTIALS, JNDI_PASSWORD); 
        JNDIEnv.put(Context.URL_PKG_PREFIXES, JNDI_URL_PKGS);
    }
    
    /**
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        DuplicateDetectionIssue client = new DuplicateDetectionIssue();
        client.publish(args);
        client.getMessageCount(args);
        client.consume(args);
    }
    
    private void getMessageCount(String[] args) throws Exception {
        String message = null;
        
        // Retrieve the ObjectName of the queue. This is used to identify the server resources to manage
        ObjectName objName = ObjectNameBuilder.DEFAULT.getJMSQueueObjectName(DuplicateDetectionIssue.DEST_NAME);

        // Create JMX Connector to connect to the server's MBeanServer
        jmxc = connectJMX(DuplicateDetectionIssue.JMX_URL, null, null);

        // Retrieve the MBeanServerConnection
        MBeanServerConnection mbsc = null;
        try {
            mbsc = jmxc.getMBeanServerConnection();
        } catch (IOException ioe) {
            message = "Caught exception when trying to find heap memory usage";
            System.err.println(message);
            ioe.printStackTrace();
            throw ioe;
        }

        // Create a JMSQueueControl proxy to manage the queue on the server
        JMSQueueControl queueControl = (JMSQueueControl)MBeanServerInvocationHandler.newProxyInstance(mbsc, objName, JMSQueueControl.class, false);

        // Display the number of messages in the queue
        System.out.println(queueControl.getName() + " contains " + queueControl.getMessageCount() + " messages");

        // We close the JMX connector
        closeJMX(jmxc);
        
    }    
   
    /**
     * @param service
     * @param user
     * @param password
     * @return
     * @throws Exception
     */
    public JMXConnector connectJMX(String service, String user, String password) throws Exception {
        String message = null;
        JMXConnector jmxc = null;
        JMXServiceURL jmxUrl = null;
        
        System.out.println("Using JMX service URL [" + service + "]");
        try {
            jmxUrl = new JMXServiceURL(service);
        } catch (MalformedURLException mfe) {
            message = "Bad URL";
            System.err.println(message);
            mfe.printStackTrace();
            throw mfe;
        }
        
        try {
            Map<String, String[]> env = new HashMap<String, String[]>();
            
            if (user != null && password != null) {
                String[] credentials = new String[] { user, password };
                env.put(JMXConnector.CREDENTIALS, credentials);
            }
            jmxc = JMXConnectorFactory.connect(jmxUrl, env);
        } catch (IOException ioe) {
            message = "Cannot create JMXConnector to service URL [" + service + "] using username [" + user + "]";
            System.err.println(message);
            ioe.printStackTrace();
            throw ioe;
        }
        return (jmxc);
    }
    
    /**
     * @throws Exception
     */
    public void closeJMX(JMXConnector jmxc) throws Exception {
        String message = null;
        
        if (jmxc == null) {
            System.err.println("Unable to close JMX connector. Null passed in.");
            return;
        }
        
        try {
            System.out.println("Attempting to close JMX connection ");
            jmxc.close();
            jmxc = null;
            System.out.println("Successfully closed JMX connection");
        } catch (IOException ioe) {
            message = "Failed to close JMX connection";
            System.err.println(message);
            ioe.printStackTrace();
            throw ioe;
        }
    } 
    
    public void consume(String[] args) {
        
        ConnectionFactory connFactory = null;
        Connection conn = null;
        Session session = null;
        MessageProducer msgProducer = null;
        MessageConsumer msgConsumer = null;
        Destination aDestination = null;
        TextMessage tm = null;
        
        try {
            InitialContext ctx = new InitialContext(JNDIEnv);            
            connFactory = (ConnectionFactory) ctx.lookup(CF_LOOKUP_NAME);
            conn = connFactory.createConnection(BROKER_USER, BROKER_PASSWORD);
            conn.setClientID("JBossPOC.SelectorTest");
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            aDestination = (Destination) ctx.lookup(DEST_LOOKUP_NAME);
            msgProducer = session.createProducer(aDestination);
            System.out.println("Using consumer destination: [" + DEST_LOOKUP_NAME + "]");
            conn.start();
            
            /* Consume all messages */
            int msgCount = 0;
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            msgConsumer = session.createConsumer(aDestination);
            System.out.println ("Starting to receive messages ....");
            while (true) {
                tm = (TextMessage) msgConsumer.receive(2*1000);
                if (tm == null) {
                    System.err.println ("No message received");
                    break;
                } else {
                    /* Create a string representation of the message properties. We will ignore properties that begin with 'JMSX'
                     * since these are added by Sun MQ and are there on all messages. */
                    String propsString = "";
                    String propName = null;
                    Enumeration propsEnumeration = tm.getPropertyNames();
                    while (propsEnumeration.hasMoreElements()) {
                        propName = (String) propsEnumeration.nextElement();
                        if (!propName.startsWith("JMSX")) {
                            propsString += "{" + propName + "=" + tm.getObjectProperty(propName).toString() + "}, ";
                        }
                    }
                    
                    tm.acknowledge();
                    System.out.println ("Received message# [" + ++msgCount + "] with id [" + tm.getJMSMessageID() + "], Props=[" + propsString + "]");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("An exception occurred");
            return;
        } finally {
            try {
                if (msgProducer != null) {
                    msgProducer.close();
                    msgProducer = null;
                    System.out.println("Closed producer...");
                }                
                if (msgConsumer != null) {
                    msgConsumer.close();
                    msgConsumer = null;
                    System.out.println("Closed consumer...");
                }
                if (session != null) {
                    session.close();
                    session = null;
                    System.out.println("Closed session...");
                }
                if (conn != null) {
                    conn.close();
                    conn = null;
                    System.out.println("Closed connection...");
                }
            } catch (JMSException jmse2) {
                jmse2.printStackTrace();
                System.err.println("An exception occurred when closing objects: " + jmse2.getMessage());
            }
        }
    }
    
    public void publish(String[] args) {
        String filePath = args[0];
        
        ConnectionFactory connFactory = null;
        Connection conn = null;
        Session session = null;
        MessageProducer msgProducer = null;
        MessageConsumer msgConsumer = null;
        Destination aDestination = null;
        TextMessage tm = null;
        
        try {
            InitialContext ctx = new InitialContext(JNDIEnv);            
            connFactory = (ConnectionFactory) ctx.lookup(CF_LOOKUP_NAME);
            conn = connFactory.createConnection(BROKER_USER, BROKER_PASSWORD);
            conn.setClientID("JBossPOC.SelectorTest");
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            aDestination = (Destination) ctx.lookup(DEST_LOOKUP_NAME);
            msgProducer = session.createProducer(aDestination);
            System.out.println("Using producer destination: [" + DEST_LOOKUP_NAME + "]");
            conn.start();
            
            /* Load the message body from the file */
            String body = loadFile(filePath);
            
            /* Publish 5000 messages */
            for (int i = 0; i < 5000; ++i) {
                tm = session.createTextMessage();
                tm.setText(body);
                msgProducer.send(tm);
                System.out.println ("Sent message# [" + (i+1) + "] with id [" + tm.getJMSMessageID() + "]");
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("An exception occurred");
            return;
        } finally {
            try {
                if (msgProducer != null) {
                    msgProducer.close();
                    msgProducer = null;
                    System.out.println("Closed producer...");
                }                
                if (msgConsumer != null) {
                    msgConsumer.close();
                    msgConsumer = null;
                    System.out.println("Closed consumer...");
                }
                if (session != null) {
                    session.close();
                    session = null;
                    System.out.println("Closed session...");
                }
                if (conn != null) {
                    conn.close();
                    conn = null;
                    System.out.println("Closed connection...");
                }
            } catch (JMSException jmse2) {
                jmse2.printStackTrace();
                System.err.println("An exception occurred when closing objects: " + jmse2.getMessage());
            }
        }
    }
    
    /**
     * @param fileName
     * @return
     * @throws Exception
     */
    private String loadFile(String fileName) throws Exception {
        String body = null;
        
        System.out.println("Loading message body from [" + fileName + "]");
        BufferedReader br = new BufferedReader(new FileReader(fileName));
        String aLine = null;
        StringBuffer bodyBuffer = new StringBuffer();
        while ((aLine = br.readLine()) != null) {
            bodyBuffer.append(aLine.trim());
        }
        br.close();
        
        /* Reset the body to be the contents of the file */
        body = bodyBuffer.toString(); 
        return (body);
    }
}
